/* @(#)49	1.10  src/bos/usr/include/dbm.h, libdbm, bos420, 9613T 2/22/94 09:43:38 */
/* 
 * COMPONENT_NAME: (LIBDBM) Data Base Management Library
 * 
 * FUNCTIONS: 
 *
 * ORIGINS: 26 
 *
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	(#)dbm.h	5.1 (Berkeley) 3/27/86
 */

#ifndef _H_DBM
#ifndef _PARAMS
#if defined(__STDC__) || defined(__cplusplus)
#define _PARAMS(ARGS) ARGS
#else
#define _PARAMS(ARGS) ()
#endif
#endif /* _PARAMS */
#define _H_DBM

#ifndef NULL
/*
 * this is lunacy, we no longer use it (and never should have
 * unconditionally defined it), but, this whole file is for
 * backwards compatability - someone may rely on this.
 */
		#undef NULL
#define	NULL    0
#endif

#include <ndbm.h>

datum	fetch _PARAMS((datum));
datum	firstkey();
datum	nextkey _PARAMS((datum));
#if 0
datum	makdatum();
datum	firsthash();
long	calchash();
long	hashinc();
#endif
#endif /* _H_DBM */
